<?php
/*
 * Original-Script von Dennis Blte 
 *   http://blog.dopefreshtight.de/artikel/von-iso-8859-1-zu-utf-8-in-php-und-mysql/
 *   
 * Angepasst an die Datenbank von TYPO3 4.0.2 von Stefan Macke
 *   http://blog.stefan-macke.de/2006/11/28/konvertieren-einer-typo3-installation-nach-utf-8/
 */

// bei grossen Datensaetzen (z.B. in tt_content) reichten bei mir 8MB Speicher nicht aus
ini_set("memory_limit", 1024 * 1024 * 32);

/****************
  Functions
****************/
function is_utf8($string) {
   // From http://w3.org/International/questions/qa-forms-utf-8.html
   return preg_match('%^(?:
         [\x09\x0A\x0D\x20-\x7E]            # ASCII
       | [\xC2-\xDF][\x80-\xBF]            # non-overlong 2-byte
       |  \xE0[\xA0-\xBF][\x80-\xBF]        # excluding overlongs
       | [\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}  # straight 3-byte
       |  \xED[\x80-\x9F][\x80-\xBF]        # excluding surrogates
       |  \xF0[\x90-\xBF][\x80-\xBF]{2}    # planes 1-3
       | [\xF1-\xF3][\x80-\xBF]{3}          # planes 4-15
       |  \xF4[\x80-\x8F][\x80-\xBF]{2}    # plane 16
   )*$%xs', $string);
}

function output($string)
{
	return is_utf8($string) ? $string : utf8_encode($string);
}

/* Da ich teilweise sehr langen Text in einigen Datenbank-Feldern habe,
 * spaltet diese Funktion diesen auf, da es sonst zu "Speicherzugriffsfehlern" kommt.
 */ 
function splitit($str)
{
	$len = 4000;
	if (strlen($str) <= $len)
	{
		return output($str);
	}
	else
	{
		$out = "";
		for ($i = 0; $i < strlen($str); $i += $len)
		{
			$out .= output(substr($str, $i, $len));
		}
		return $out;
	}	

}


/****************
  SQL Link
****************/
require "class.sql_link.php";

$sql = new SQL_link('localhost', 'user', 'pass', 'db');


/****************
  DB Convert
****************/
#$tables = $sql->query_array("SHOW TABLES"); 

/* Hier habe ich nur die Tabellen eingetragen, die bei meiner TYPO3-Installation
 * konvertiet werden mussten. Die Index-Tabellen habe ich z.B. komplett auen vor
 * gelassen. Beim Eintragen neuer Tabellen darauf achten, dass diese die Schlssel-
 * spalte "uid" haben mssen!
 */
$tables = array( 
	array('be_groups'),
	array('be_users'),
	array('fe_groups'),
	array('fe_users'),
	array('pages'),
	array('static_countries'),
	array('static_country_zones'),
	array('static_currencies'),
	array('static_languages'),
	array('static_taxes'),
	array('static_template'),
	array('static_territories'),
	array('static_tsconfig_help'),
	array('sys_filemounts'),
	array('sys_history'),
	array('sys_lockedrecords'),
	array('sys_log'),
	array('sys_template'),
	array('tt_content'),
	array('tx_chcforum_category'),
	array('tx_chcforum_conference'),
	array('tx_chcforum_forumgroup'),
	array('tx_chcforum_post'),
	array('tx_chcforum_thread'),
	array('tx_templavoila_datastructure'),
	array('tx_templavoila_tmplobj')
);

$final_changes = "";

foreach($tables as $table) 
{
	
	$columns = $sql->get_columns($table[0]);
	
	$affected_columns = array();
	
	// Betroffene Spalten suchen
	foreach($columns as $column)
		if(strpos($column['Type'], "char") != false || strpos($column['Type'], "text") != false)
			$affected_columns[] = $column['Field'];
	
	// Konvertierung notwendig?
	$count_cols = count($affected_columns);
	
	if($count_cols > 0) 
	{
		// Aenderungen vorbereiten
		$load = "";
		
		foreach($affected_columns as $col)
			$load .= ", " . $col;
		
		// Anzahl Stze ermitteln
		$numrows_result = $sql->query_array("SELECT count(uid) as numrows FROM " . $table[0]);
		$numrows = intval($numrows_result[0]['numrows']);
		echo $table[0] . " " . $numrows . "\n";

		// wenn eine groe Anzahl an Stzen vorliegt, Query auf mehrere Abfragen aufspalten (sonst gibt's ein Speicherproblem)
		if ($numrows > 2000)
		{
			for ($z = 0; $z <=$numrows; $z+=2000)
			{
				echo "  " . $z . "\n";
				$rows = $sql->query_array("SELECT uid" . $load . " FROM " . $table[0] . " ORDER BY uid LIMIT $z, 2000;");
		
				// Aenderungen durchfuehren
				foreach($rows as $row) 
				{
					$changes = "";			
			
					for($i=0; $i<$count_cols; $i++) 
					{
						$base = $affected_columns[$i] . " = '" . splitit($row[$affected_columns[$i]]) . "'";
						$changes .= ($i == ($count_cols-1)) ? $base : $base . ", ";
					}
					$sql->query("UPDATE " . $table[0] . " SET " . $changes . " WHERE uid = " . $row['uid'] . ";");
				}

			}
		}

		// alles in einem Rutsch konvertieren
		else
		{
			$rows = $sql->query_array("SELECT uid" . $load . " FROM " . $table[0]);
		
			// Aenderungen durchfuehren
			foreach($rows as $row) 
			{
				$changes = "";			
				
				for($i=0; $i<$count_cols; $i++) 
				{
					$base = $affected_columns[$i] . " = '" . splitit($row[$affected_columns[$i]]) . "'";
					$changes .= ($i == ($count_cols-1)) ? $base : $base . ", ";
				}
				$sql->query("UPDATE " . $table[0] . " SET " . $changes . " WHERE uid = " . $row['uid'] . ";");
			}
		}
	} // $count_cols > 0
} // foreach

?>
